﻿using UnityEngine;
using System.Collections;
using Sirenix.OdinInspector;

namespace RgbRepair.Audio {
	public enum EBgm {
		MainBGM
	}

	public enum ESfx {
		DoorOpen,
		DoorClose,
		Footstep,
		Hologram,
		Right,
		Wrong
	}

	public class AudioManager : MonoBehaviour {
		[SerializeField]
		private AudioSource[] BGMList;
		[SerializeField]
		private AudioSource[] SFXList;

		/// <summary>
		/// Start playing a BGM from the BGMList
		/// </summary>
		/// <param name="bgmKey">The enumerator listing individual BGMs</param>
		/// <param name="changeLooping">Should the BGM loop? Using null will use its initial loop state</param>
		public void PlayBGM (EBgm bgmKey, bool? changeLooping = null) {
			int index = (int) bgmKey;
			
			if (index >= BGMList.Length) {
				return;
			}

			BGMList[index].Play ();
			if (changeLooping.HasValue) {
				BGMList[index].loop = changeLooping.Value;
			}
		}

		public void StopBGM (EBgm bgmKey) {
			int index = (int) bgmKey;

			if (index >= BGMList.Length) {
				return;
			}

			BGMList[index].Stop ();
		}

		/// <summary>
		/// Start playing a SFX from the SFXList
		/// </summary>
		/// <param name="sfxKey">The enumerator listing individual SFXs</param>
		/// <param name="changeLooping">Should the SFX loop? Using null will use its initial loop state</param>
		public void PlaySFX (ESfx sfxKey, bool? changeLooping = null) {
			int index = (int) sfxKey;

			if (index >= SFXList.Length) {
				return;
			}

			SFXList[index].Play ();
			if (changeLooping.HasValue) {
				SFXList[index].loop = changeLooping.Value;
			}
		}

		public void StopSFX (ESfx sfxKey) {
			int index = (int) sfxKey;

			if (index >= SFXList.Length) {
				return;
			}

			SFXList[index].Stop ();
		}

		public void StartFootstep () {
			PlaySFX (ESfx.Footstep, true);
		}

		public void StopFootstep () {
			StopSFX (ESfx.Footstep);
		}

		public void PlayHologramSFX () {
			PlaySFX (ESfx.Hologram);
		}

		public void PlayCorrectSFX () {
			PlaySFX (ESfx.Right);
		}

		public void PlayIncorrectSFX () {
			PlaySFX (ESfx.Wrong);
		}
	}
}
