using System.IO;
using System.Threading.Tasks;
using UnityEngine;
using Seven;
using System;

namespace RGBRepair{
    public static class CameraManager {
        public static event Action<Texture2D> onTakePicture;
        public static async Task<Texture2D> TakePicture()
        {
            var permission =  NativeCamera.CheckPermission();

            while (permission == NativeCamera.Permission.ShouldAsk || permission == NativeCamera.Permission.Denied)
            {
                switch(permission)
                {
                    case NativeCamera.Permission.ShouldAsk:
                    {
                        permission =  NativeCamera.RequestPermission();
                        break;
                    }

                    case NativeCamera.Permission.Denied:
                    {
                        var canOpenSetting = NativeCamera.CanOpenSettings();
                        if(canOpenSetting)
                            NativeCamera.OpenSettings();

                        break;
                    }
                }
            }

            if(permission != NativeCamera.Permission.Granted)
                return null;

            var path = string.Empty;
            var complete = false;
            NativeCamera.TakePicture((x) => 
            {
                path = x;
                complete = true;
            });

            await new WaitUntil(()=> complete);

            if(path.IsNullOrEmpty())
                return null;

            var byteArray = File.ReadAllBytes(path);
            Texture2D texture = new Texture2D(8, 8,TextureFormat.ARGB32,false);
            texture.LoadImage(byteArray);
            texture.Apply();
            onTakePicture?.Invoke(texture);
            return texture;
        } 
    }
}