﻿using UnityEngine;
using UnityEngine.UI;
using RgbRepair.ColorProcessing;
using System.Collections;
using System.Collections.Generic;

public class ColorTesting : MonoBehaviour {
	[SerializeField]
	private Image image;
	[SerializeField]
	private Text debugText;
	[SerializeField]
	private Text debugText2;
	[SerializeField]
	private Button processButton;
	[SerializeField]
	private float moreThanThreshold = 0.5f;
	[SerializeField]
	private float lessThanThreshold = 0.5f;
	[SerializeField]
	private float maxErrorPercentage = 0.7f;
	//[SerializeField]
	private ColorCriteriaEntry[] colorCriteriaLibrary;

	private void Start () {
		processButton.onClick.AddListener (ProcessImage);

		colorCriteriaLibrary = new ColorCriteriaEntry[] {
			new ColorCriteriaEntry (
					EColor.Red,
					new ColorCriterion[] {
						new ColorCriterion (EColorPart.Red, EColorComparator.MoreThan, moreThanThreshold),
						new ColorCriterion (EColorPart.Green, EColorComparator.LessThan, lessThanThreshold),
						new ColorCriterion (EColorPart.Blue, EColorComparator.LessThan, lessThanThreshold)
					}
				),
			new ColorCriteriaEntry (
					EColor.Blue,
					new ColorCriterion[] {
						new ColorCriterion (EColorPart.Red, EColorComparator.LessThan, lessThanThreshold),
						new ColorCriterion (EColorPart.Green, EColorComparator.LessThan, lessThanThreshold),
						new ColorCriterion (EColorPart.Blue, EColorComparator.MoreThan, moreThanThreshold)
					}
				),
			new ColorCriteriaEntry (
					EColor.Green,
					new ColorCriterion[] {
						new ColorCriterion (EColorPart.Red, EColorComparator.LessThan, lessThanThreshold),
						new ColorCriterion (EColorPart.Green, EColorComparator.MoreThan, moreThanThreshold),
						new ColorCriterion (EColorPart.Blue, EColorComparator.LessThan, lessThanThreshold)
					}
				),
			new ColorCriteriaEntry (
					EColor.Yellow,
					new ColorCriterion[] {
						new ColorCriterion (EColorPart.Red, EColorComparator.MoreThan, moreThanThreshold),
						new ColorCriterion (EColorPart.Green, EColorComparator.MoreThan, moreThanThreshold),
						new ColorCriterion (EColorPart.Blue, EColorComparator.LessThan, lessThanThreshold)
					}
				),
			new ColorCriteriaEntry (
					EColor.Purple,
					new ColorCriterion[] {
						new ColorCriterion (EColorPart.Red, EColorComparator.MoreThan, moreThanThreshold),
						new ColorCriterion (EColorPart.Green, EColorComparator.LessThan, lessThanThreshold),
						new ColorCriterion (EColorPart.Blue, EColorComparator.MoreThan, moreThanThreshold)
					}
				),
			new ColorCriteriaEntry (
					EColor.White,
					new ColorCriterion[] {
						new ColorCriterion (EColorPart.Red, EColorComparator.MoreThan, moreThanThreshold),
						new ColorCriterion (EColorPart.Green, EColorComparator.MoreThan, moreThanThreshold),
						new ColorCriterion (EColorPart.Blue, EColorComparator.MoreThan, moreThanThreshold)
					}
				),
			new ColorCriteriaEntry (
					EColor.Black,
					new ColorCriterion[] {
						new ColorCriterion (EColorPart.Red, EColorComparator.LessThan, lessThanThreshold),
						new ColorCriterion (EColorPart.Green, EColorComparator.LessThan, lessThanThreshold),
						new ColorCriterion (EColorPart.Blue, EColorComparator.LessThan, lessThanThreshold)
					}
				)
		};
	}

	public void ProcessImage () {
		Texture2D texture = image.sprite.texture;
		Color[] centerPixels = GetCenterPixels (texture, 20);

		string debugTextStr = "";
		foreach (ColorCriteriaEntry criteriaEntry in colorCriteriaLibrary) {
			bool res = ColorProcessingManager.DoPixelsFitCriteria (centerPixels, criteriaEntry.criteria, maxErrorPercentage);
			if (res) {
				debugTextStr += " Is " + criteriaEntry.colorName.ToString();
			} else {
				debugTextStr += " Is not " + criteriaEntry.colorName.ToString();
			}
			debugTextStr += "\r\n";
		}
		debugText.text = debugTextStr;

		Color centerPixel = GetCenterPixel (texture);
		debugText2.text = "r" + centerPixel.r + " g" + centerPixel.g + " b" + centerPixel.b;
	}

	private Color[] GetCenterPixels (Texture2D tex, int radius) {
		int centerX = tex.width / 2;
		int centerY = tex.height / 2;
		int resultSize = (radius * 2 + 2) * (radius * 2 + 2);
		List<Color> resultPixels = new List<Color> ();
		for (int i = centerX - radius; i <= centerX + radius; i++) {
			for (int j = centerY - radius; j <= centerY + radius; j++) {
				resultPixels.Add (tex.GetPixel (i, j));
			}
		}
		return resultPixels.ToArray ();
	}

	private Color GetCenterPixel (Texture2D tex) {
		int centerX = tex.width / 2;
		int centerY = tex.height / 2;
		return tex.GetPixel (centerX, centerY);
	}
}
