using System.Collections.Generic;
using Sirenix.OdinInspector;
using UnityEngine;
using System;
using Seven;
using DG.Tweening;
using System.Threading.Tasks;
using UnityEngine.Rendering;

namespace RGBRepair
{
    [HideMonoScript]
    public class Customer : SerializedMonoBehaviour, iCustomer,IDisposable
    {
        #region const
        const string grpSetting = "Setting";
        const string grpRuntime = "Runtime";
        const string  grpCompoenent = "Component";
        #endregion

        #region Field
        [SerializeField,FoldoutGroup(grpSetting)]
        private string _id;

        [SerializeField,InlineEditor,FoldoutGroup(grpSetting)]
        private List<iItem> _items;

        [SerializeField,FoldoutGroup(grpCompoenent)]
        private Animator _animator;
        
        [SerializeField,FoldoutGroup(grpCompoenent)]
        private SortingGroup _sorting;
        #endregion

        #region private
        private iItem _currentItem;
        #endregion

        
        #region iCustomer
		public event Action onWalkStarted;
		public event Action onWalkEnded;
		public event Action onItemSpawned;
        public event Action<iCustomer> onComplete;
        public string id => _id;

        public List<iItem> items => _items;
        
        [ShowInInspector,FoldoutGroup(grpRuntime)]
        public iItem currentItem => _currentItem;
     
        public bool isCompleted {
            get
            {
                if(_items.IsNullOrEmpty())
                    return false;

                var result = true;
                foreach (var item in _items)
                {
                    if(item.IsNull())
                        continue;

                    result &= item.isComplete;

                    if(result == false)
                        break;
                }

                return result;
            }
        }

        [Button]
        public async Task Play()
        {
            if(_animator) 
            {
                _animator.SetTrigger("play");
				onWalkStarted?.Invoke ();
                await new WaitForEndOfFrame();
                var time = _animator.GetCurrentAnimatorStateInfo(0).length;
                await new WaitForSeconds(time);
				onWalkEnded?.Invoke ();
			}

            var item = RandomLoadItem();
            SetCurrentItem(item);
            item.onComplete += OnItemComplete;
            item.onFailed += OnItemFailed;
            await item.Play();
        }
        
        #endregion

        #region IDisposable
        public void Dispose()
        {
            GameObject.Destroy(gameObject);

            if(!_currentItem.IsNull())
                _currentItem.Dispose(); 

            _currentItem = null;
        }
        #endregion

        [Button]
        private async Task Done()
        {
            if(_animator) 
            {
                _animator.SetTrigger("done");
				onWalkStarted?.Invoke ();
                await new WaitForEndOfFrame();
                var time = _animator.GetCurrentAnimatorStateInfo(0).length;
                await new WaitForSeconds(time);
				onWalkEnded?.Invoke ();
			}
        }      
        private iItem RandomLoadItem()
        {
            var randomIndex = UnityEngine.Random.Range(0,_items.Count);
            var prefab = _items[randomIndex];
            var item = ItemFactory.Create(prefab);
            return item;
        }
        private void SetCurrentItem(iItem item){
            this._currentItem = item;
        }

        public void SetLayerOrder(int index)
        {
            _sorting.sortingOrder = index;
        }


        private void Sad()
        {
            _animator.SetTrigger("sad");
        }
        private void Happy()
        {
            _animator.SetTrigger("happy");
        }
        private void Netral()
        {
            _animator.SetTrigger("netral");
        }
        
        public void OpenDoor(){
            Environment.instance.OpenDoor();
        }
        public void CloseDoor(){
            Environment.instance.CloseDoor();
        }
        
        private void OnDestroy() {
            onComplete = null;
        }

        #region Callback
        private async void OnItemComplete(iItem item)
        {
            Happy();
            await new WaitForSeconds(1);
            await Done();
            "OnCustomer Complete".Log();
            onComplete?.Invoke(this);
        }
        private async void OnItemFailed(iItem item)
        {
           _currentItem.HideHologram();
           Sad();
           await new WaitForSeconds(2);
           Netral();
           _currentItem.ShowHologram();
        }
        #endregion
    }
}