using Sirenix.OdinInspector;
using UnityEngine;
using System;
using DG.Tweening;
using System.Threading.Tasks;
using Seven;
using RgbRepair.ColorProcessing;

namespace RGBRepair{

    [HideMonoScript]
    public class Item : SerializedMonoBehaviour ,iItem,IDisposable
    {
        #region const
        const string grp = "Setting";
        const string grpComponent = "Component";
        #endregion

        #region Field
        [SerializeField,InlineEditor,FoldoutGroup(grp)]
        private iMaterial _required;

        [SerializeField,FoldoutGroup(grp)]
        private int _coin = 1;

        [SerializeField,FoldoutGroup(grpComponent)]
        private Animator _animator;

        [SerializeField,FoldoutGroup(grpComponent)]
        private Transform _materialPlaceHolder;
        #endregion

        #region Private
        private bool _isComplete;
        #endregion

        #region iItem
        public event Action<iItem> onComplete;
        public event Action<iItem> onFailed;
		public event Action onItemCorrect;
		public event Action onItemIncorrect;
		public event Action onHologram;

		public iMaterial required => _required;
        public bool isComplete => _isComplete;
        public int coin => _coin;

        private void Start() 
        {
            var material = MaterialFactory.Create(_required);
            material.gameObject.transform.SetParent(_materialPlaceHolder.transform);  
            material.gameObject.transform.position = _materialPlaceHolder.position;  
            material.gameObject.transform.localScale = Vector3.one * 1.5f;
            _required = material;
        }
        public async Task Play()
        {
            if(_animator) 
            {
                _animator.SetTrigger("play");
                await new WaitForEndOfFrame();
                var time = _animator.GetCurrentAnimatorStateInfo(0).length;
                await new WaitForSeconds(time);
                ShowHologram();
            }
            GameManager.instance.OnReadyUseCamera(true);
            CameraManager.onTakePicture += OnCameraTakePicture;
        }
        [Button]
        public async Task Done()
        {       
            HideHologram();
            GameManager.instance.OnReadyUseCamera(false);
			
            onItemCorrect?.Invoke ();
			if (_animator) 
            {
                _animator.SetTrigger("done");
                await new WaitForEndOfFrame();
                var time = _animator.GetCurrentAnimatorStateInfo(0).length;
                await new WaitForSeconds(time);
            }
            OnComplete();
        }
        [Button]
        public void Failed()
        {
			onItemIncorrect?.Invoke ();
            OnFailed();
        }
        #endregion

        #region iDispose
        public void Dispose()
        {
            GameObject.Destroy(gameObject);
        }
        #endregion

        public void ShowHologram()
        {
			onHologram?.Invoke ();
            _materialPlaceHolder.gameObject.SetActive(true);
        }
        public void HideHologram()
        {
			onHologram?.Invoke ();
            _materialPlaceHolder.gameObject.SetActive(false);
        }

        private void OnDestroy() 
        {
            onComplete = null;
            onFailed = null;    
            CameraManager.onTakePicture -= OnCameraTakePicture;
        }
        private void OnMouseDown() {
            "MouseDown".Log();
        }

        #region Callback
        private void OnComplete()
        {
            _isComplete = true;
            onComplete?.Invoke(this);
        }
        private void OnFailed(){
            _isComplete = false;
            onFailed?.Invoke(this);
        }
        private async void OnCameraTakePicture(Texture2D texture)
        {
			EColor mainColor = ColorProcessingManager.GetMainColorOfTexture (texture, 20, 0.5f);
			EColor requiredColor = ColorProcessingManager.ConvertMaterialToEColor (required);
			if (mainColor.Equals (requiredColor)) {
				await Done ();
			} else {
				Failed ();
			}
        }
        #endregion
    }
}