using UnityEngine;
using Sirenix.OdinInspector;
using System;
using System.Collections.Generic;
using Seven;
using DG.Tweening;
using System.Threading.Tasks;
using RgbRepair.Audio;

namespace RGBRepair
{
    [HideMonoScript]
    public class LevelManager : SerializedMonoBehaviour, iLevelManager
    {
        #region const
        const string grpSetting = "Setting";
        const string grpRuntime = "Runtime";
        #endregion

        #region Field
        [SerializeField,FoldoutGroup(grpSetting)]
        private List<iCustomer> _customers;

		[SerializeField, FoldoutGroup (grpSetting)]
		private AudioManager _audioManager;
        #endregion
 
        #region Private
        private iCustomer _currentCustomer;
        private iGameManager _gameManager;
        private List<iCustomer> _customersCompleted = new List<iCustomer>();
        private bool _inited;
        #endregion

        #region iLevelManager

        [ShowInInspector,FoldoutGroup(grpRuntime)]
        public iCustomer currentCustomer => _currentCustomer;
        public List<iCustomer> customers => _customers;

        public event Action<iCustomer> onCustomerComplete;
        public event Action<iItem> onItemComplete;

        public iCustomer GetCustomerPrefab(int index)
        {
            if(index < 0 )
                return null;

            if(index > _customers.Count - 1)
                return null;

            return _customers[index];
        }

        public void Init(iGameManager gameManager)
        {
            if(_inited)
                return;

            this._gameManager = gameManager;
        
            _inited = true;

			_audioManager.PlayBGM (EBgm.MainBGM, true);
        }
        public async Task Load(iCustomer prefab)
        {   
            //Unload prev Customer
            if(!_currentCustomer.IsNull())
            {
                await UnLoad(_currentCustomer);
                _currentCustomer.Dispose();
                _currentCustomer = null;
            }
           

            if(prefab.IsNull())
            {
                SetCurrentCustomer(null);
                return;
            }
                
            //Load new customer
            var customer =  CutomerFactory.Create(prefab); 
            SetCurrentCustomer(customer);
			if (!customer.IsNull ()) {
				await customer.Play ();
				if (!customer.currentItem.IsNull()) {
					customer.currentItem.onHologram += _audioManager.PlayHologramSFX;
					customer.currentItem.onItemCorrect += _audioManager.PlayCorrectSFX;
					customer.currentItem.onItemIncorrect += _audioManager.PlayIncorrectSFX;
                    customer.currentItem.onComplete += onItemComplete;
                }
			}
            customer.onComplete += OnCustomerComplete;  
        }

        public async Task UnLoad(iCustomer customer)
        {
        
            if(customer.IsNull())
                return;

             customer.Dispose();
             customer = null;
        }
        #endregion

        private void SetCurrentCustomer(iCustomer customer){
            this._currentCustomer = customer;

            if(customer.IsNull())
                return;
                
			customer.onWalkStarted += _audioManager.StartFootstep;
			customer.onWalkEnded += _audioManager.StopFootstep;
        }
    
        private void OnCustomerComplete(iCustomer customer)
        {
            if(_customersCompleted.Find(x=> x.id == customer.id) == null)
                _customersCompleted.Add(customer);
            
            onCustomerComplete?.Invoke(customer);
        }

        public iCustomer GetRandomCustomer()
        {
            var randomIndex = UnityEngine.Random.Range(0,_customers.Count);
            return _customers[randomIndex];


            // foreach (var customer in _customers)
            // {
            //    var find = _customersCompleted.Find(x=> x.id == customer.id);
            //    if(find.IsNull())
            //    {
            //        return customer;
            //    }        
            // }

            // return null;
        }
    }
}