using UnityEngine;
using Sirenix.OdinInspector;

public abstract class Singleton<T> : SerializedMonoBehaviour where T : Component
{	
	#region Fields
	protected static T _instance;
	protected virtual bool dontDestroyOnLoad {
		get{
			return true;
		}
	}
	#endregion

	#region Properties
	public static T instance
	{
		get
		{
			if ( _instance == null )
			{
				_instance = FindObjectOfType<T> ();
				if ( _instance == null )
				{
					GameObject obj = new GameObject ();
					obj.name = typeof ( T ).Name;
					_instance = obj.AddComponent<T> ();
				}
			}
			return _instance;
		}
	}
	#endregion

	protected virtual void Awake ()
	{
		if ( _instance == null )
		{
			_instance = this as T;
			if(dontDestroyOnLoad) 
			{
				DontDestroyOnLoad ( gameObject );
			}
		}
		else
		{
			Destroy ( gameObject );
		}
    }
	protected virtual void OnDestroy() {
		if(_instance == this)
			_instance = null;
	}
}