﻿// Copyright (c) 2015 - 2019 Doozy Entertainment / Marlink Trading SRL. All Rights Reserved.
// This code can only be used under the standard Unity Asset Store End User License Agreement
// A Copy of the EULA APPENDIX 1 is available at http://unity3d.com/company/legal/as_terms

using System;
using UnityEngine;
using UnityEngine.Rendering;
using Language = Doozy.Engine.Language;  

// ReSharper disable InconsistentNaming

namespace Doozy.Engine.Utils
{
    [Serializable]
    public class UILanguagePack : LanguagePack
    {
        private static Engine.Language s_loadedLanguage = Engine.Language.Unknown;
        private static UILanguagePack s_instance;

        public static UILanguagePack Instance
        {
            get
            {
                if (s_instance != null && s_loadedLanguage == CurrentLanguage) return s_instance;
                s_instance = Resources.Load<UILanguagePack>(typeof(UILanguagePack).Name + CurrentLanguage);
                s_loadedLanguage = CurrentLanguage;
                return s_instance;
            }
        }

        public Engine.Language TargetLanguage = DEFAULT_LANGUAGE;

        [Header("Labels")]
        public string Action = "Action";
        public string Actions = "Actions";
        public string ActivateLoadedScenesNodeName = "Activate Loaded Scenes";
        public string ActiveNode = "Active Node";
        public string ActiveSceneChange = "Active Scene Change";
        public string AddCategory = "Add Category";
        public string AddItem = "Add Item";
        public string AddSounds = "Add Sounds";
        public string AddSource = "Add Source";
        public string AddSymbolToAllBuildTargetGroups = "Add symbol to all the BuildTargetGroups?";
        public string AddToPopupQueue = "Add to Popup Queue";
        public string After = "After";
        public string AfterDelay = "After Delay";
        public string AllowMultipleClicks = "Allow Multiple Clicks";
        public string AllowSceneActivation = "Allow Scene Activation";
        public string AllSoundsBeforePlaying = "all sounds before playing";
        public string AlternateInput = "Alternate Input";
        public string AlternateKeyCode = "Alternate Key Code";
        public string AlternateVirtualButton = "Alternate Virtual Button";
        public string AnimateAllUIViewsWithSameCategoryAndName = "Animate all UIViews with the same category & name";
        public string AnimateValue = "Animate Value";
        public string Animation = "Animation";
        public string AnimationCurve = "Animation Curve";
        public string AnimationType = "Animation NamesDatabaseType";
        public string Animator = "Animator";
        public string AnimatorEvents = "Animator Events";
        public string Animators = "Animators";
        public string AnotherEntryExists = "There is another entry with the same name.";
        public string AnyButton = "Any Button";
        public string AnyGameEvent = "Any GameEvent";
        public string AnyGameEventWillTriggerThisListener = "Any game event will trigger this listener";
        public string AnyScene = "Any Scene";
        public string AnyUIButtonWillTriggerThisListener = "Any UIButton will trigger this listener";
        public string AnyUIDrawerWillTriggerThisListener = "Any UIDrawer will trigger this listener";
        public string AnyUIViewWillTriggerThisListener = "Any UIView will trigger this listener";
        public string ApplicationQuit = "Application Quit";
        public string AreYouSureConvertToGraph = "Are you sure you want to convert this SubGraph into a Graph?";
        public string AreYouSureConvertToSubGraph = "Are you sure you want to convert this Graph into a SubGraph";
        public string AreYouSureYouWantToDeleteDatabase = "Are you sure you want to delete the database?";
        public string AreYouSureYouWantToDeletePopupReference = "Are you sure you want to delete the popup reference?";
        public string AreYouSureYouWantToRemoveCategory = "Are you sure you want to remove the category?";
        public string AreYouSureYouWantToRemoveTheEntry = "Are you sure you want to remove the database entry?";
        public string AreYouSureYouWantToResetDatabase = "Are you sure you want to reset this database?";
        public string Arrow = "Arrow";
        public string ArrowComponents = "Arrow Components";
        public string AtFinish = "@FINISH";
        public string AtStart = "@START";
        public string AudioClip = "Audio Clip";
        public string AutoDisableUIInteractionsDescription = "Automatically disable UI interactions when animating views";
        public string AutoHideAfterShow = "Auto Hide after Show";
        public string AutoKillIdleControllers = "Auto Kill Idle Controllers";
        public string AutoKillIdleControllersDescription = "Auto kill any Soundy Controllers that are have been unused for the set idle kill duration";
        public string AutoRebuild = "Auto Rebuild";
        public string AutoResetSequence = "Auto Reset Sequence";
        public string AutoSelectButtonAfterShow = "Auto selected button after Show";
        public string AutoSort = "Auto Sort";
        public string AutoStartLoopAnimation = "Auto Start Loop Animation";
        public string BackButton = "Back Button";
        public string BackButtonNodeName = "Back Button";
        public string Behavior = "Behavior";
        public string BehaviorAtStart = "Behavior at Start";
        public string Behaviors = "Behaviors";
        public string BlockBackButton = "Block Back Button";
        public string BorderWidth = "Border Width";
        public string BuildIndex = "Build Index";
        public string ButtonCategory = "Button Category";
        public string ButtonLabel = "Button Label";
        public string ButtonName = "Button Name";
        public string Buttons = "UIButton";
        public string Cancel = "Cancel";
        public string CannotAddEmptyEntry = "Cannot add an empty entry.";
        public string Canvas = "Canvas";
        public string CanvasName = "Canvas Name";
        public string Category = "Category";
        public string CategoryIsEmpty = "Category is empty...";
        public string Center = "Center";
        public string CenterSelectedNodes = "Center Selected Nodes";
        public string Chance = "Chance";
        public string CheckingForIssues = "Checking for issues";
        public string ChildHeight = "Child Height";
        public string ChildHeightFactor = "Child Height Factor";
        public string ChildRotation = "Child Rotation";
        public string ChildWidth = "Child Width";
        public string ChildWidthFactor = "Child Width Factor";
        public string Clear = "Clear";
        public string ClearRecentOpenedGraphsList = "Clear Recent Graphs";
        public string ClearRecentOpenedGraphsListDescription = "Clears the list of recently opened graphs";
        public string ClearSearch = "Clear Search";
        public string ClickAnywhere = "Click Anywhere";
        public string ClickContainer = "Click Container";
        public string ClickMode = "Click Mode";
        public string ClickOverlay = "Click Overlay";
        public string Clockwise = "Clockwise";
        public string Close = "Close";
        public string Closed = "Closed";
        public string CloseDirection = "Close Direction";
        public string ClosedPosition = "Closed Position";
        public string CloseDrawer = "Close Drawer";
        public string CloseSpeed = "Close Speed";
        public string Color = "Color";
        public string Compiling = "Compiling";
        public string ComponentDisabled = "Component disabled";
        public string Connected = "Connected";
        public string ConnectionPoint = "Connection Point";
        public string Connections = "Connections";
        public string Container = "Container";
        public string ContainerSize = "Container Size";
        public string Continue = "Continue";
        public string ControlChildHeight = "Control Child Height";
        public string ControlChildWidth = "Control Child Width";
        public string ControllerIdleKillDuration = "Controller Idle Kill Duration";
        public string ControllerIdleKillDurationDescription = "The duration (in seconds) a Soundy Controller needs to be idle to be considered killable";
        public string ControllerName = "Controller Name";
        public string ConvertToGraph = "Convert to Graph";
        public string ConvertToSubGraph = "Convert to SubGraph";
        public string Copy = "Copy";
        public string Create = "Create";
        public string CreateAnimation = "Create Animation";
        public string CreateGraph = "Create Graph";
        public string CreateNewCategory = "Create a new category";
        public string CreateNewGraph = "Create a new Graph";
        public string CreateNewGraphAsSubGraph = "Create a new Graph as a SubGraph";
        public string CreateNode = "Create Node";
        public string CreateParentAndCenterPivot = "Create Parent and Center Pivot";
        public string CreateSubGraph = "Create SubGraph";
        public string CurrentTimeScale = "Current Time Scale";
        public string Custom = "Custom";
        public string CustomName = "Custom Name";
        public string CustomPosition = "Custom Position";
        public string CustomResetValue = "Custom Reset Value";
        public string CustomSortingLayer = "Custom Sorting Layer";
        public string CustomSortingOrder = "Custom Sorting Order";
        public string CustomStartPosition = "Custom Start Position";
        public string Cut = "Cut";
        public string Database = "Database";
        public string DatabaseAlreadyExists = "Database name already exists. Enter a new name and try again.";
        public string DatabaseHasBeenReset = "Database has been reset";
        public string DatabaseIsEmpty = "Database is empty...";
        public string DatabaseName = "Database Name";
        public string DatabaseRefreshed = "Database Refreshed";
        public string DatabaseSorted = "Database Sorted";
        public string Debug = "Debug";
        public string DebugMode = "Debug Mode";
        public string DefaultDotAnimationSpeedDescription = "Animation speed for the dot that travels across connections";
        public string DefaultValues = "Default Values";
        public string DefaultValuesDescription = "Default values for newly created components";
        public string DefaultZoom = "Default Zoom";
        public string DefaultZoomDescription = "Default zoom value used when creating a new graph";
        public string Delete = "Delete";
        public string DeleteDatabase = "Delete Database";
        public string DeletedPopupReference = "Deleted Popup Reference";
        public string DeletePopupReference = "Delete Popup Reference";
        public string DeletePreset = "Delete Preset";
        public string Description = "Description";
        public string Deselect = "Deselect";
        public string DeselectAnyButton = "If any button is selected, deselect it on";
        public string DeselectButton = "Deselect Button";
        public string DeselectButtonAfterClick = "Deselect Button After Click";
        public string DestroyAfterHide = "Destroy After Hide";
        public string DetectGestures = "Detect Gestures";
        public string Direction = "Direction";
        public string Disable = "Disable";
        public string DisableButtonInterval = "Disable Button Interval";
        public string DisableCanvas = "Disable Canvas";
        public string Disabled = "Disabled";
        public string DisableFunctionality = "Disable Functionality";
        public string DisableGameObject = "Disable GameObject";
        public string DisableInterval = "Disable Interval";
        public string DisablePlugin = "Disable Plugin";
        public string DisableTriggerAfterActivation = "Disable Trigger After Activation";
        public string Disconnect = "Disconnect";
        public string DispatchButtonClicks = "Dispatch Button Clicks";
        public string DispatchGameEvents = "Dispatch Game Events";
        public string DisplayTarget = "Display Target";
        public string DontDestroyGameObjectOnLoad = "Don't destroy GameObject on scene change";
        public string DotAnimationSpeed = "Dot Animation Speed";
        public string Down = "Down";
        public string DragDrawer = "Drag Drawer";
        public string DrawerName = "Drawer Name";
        public string DropAudioClipsHere = "Drop AudioClips Here";
        public string Duration = "Duration";
        public string Ease = "Ease";
        public string EaseType = "Ease NamesDatabaseType";
        public string Effect = "Effect";
        public string Elasticity = "Elasticity";
        public string Email = "Email";
        public string Empty = "Empty";
        public string Enable = "Enable";
        public string Enabled = "Enabled";
        public string EnableFunctionality = "Enable Functionality";
        public string EnablePlugin = "Enable Plugin";
        public string EnableSupportForMasterAudio = "Enable support for MasterAudio in the Control Panel";
        public string EnterCategoryName = "Enter a category name";
        public string EnterDatabaseName = "Enter a database name";
        public string EnterGameEventToListenFor = "Enter a game event to listen for";
        public string EnterNodeName = "Enter";
        public string Error = "Error";
        public string Event = "Event";
        public string EveryLongTapWillTriggerThisListener = "Every LongTap will trigger this listener";
        public string EverySwipeWillTriggerThisListener = "Every Swipe will trigger this listener";
        public string EveryTapWillTriggerThisListener = "Every Tap will trigger this listener";
        public string ExitNodeName = "Exit";
        public string ExposedParameterName = "Exposed Parameter Name";
        public string Fade = "Fade";
        public string FadeBy = "Fade By Alpha Value (0-1)";
        public string FadeFrom = "Fade From Alpha Value (0-1)";
        public string FadeOutContainer = "Fade Out Container";
        public string FadeTo = "Fade To Alpha Value (0-1)";
        public string Feather = "Feather";
        public string FeatherExpandsSize = "Feather Expands Size";
        public string FillAmount = "Fill Amount";
        public string FillCenter = "Fill Center";
        public string FillMethod = "Fill Method";
        public string FillOrigin = "Fill Origin";
        public string FixedSize = "FixedSize";
        public string FsmName = "FSM Name";
        public string Functionality = "Functionality";
        public string FunctionalityDescription = "Toggle system capabilities";
        public string GameEvent = "Game Event";
        public string GameEvents = "Game Events";
        public string GameObject = "Game Object";
        public string GeneralSettings = "General Settings";
        public string GestureType = "Gesture Type";
        public string GetInTouch = "Get in touch";
        public string GetPosition = "Get Position";
        public string GetSceneBy = "Get Scene By";
        public string GlobalListener = "Global Listener";
        public string GoToEnterNode = "Go to OnEnterNode Node";
        public string GoToExitNode = "Go to OnExitNode Node";
        public string GoToStartNode = "Go to Start Node";
        public string Graph = "Graph";
        public string GraphHasNoNodes = "Graph has no nodes";
        public string GraphicRaycaster = "Graphic Raycaster";
        public string GraphId = "Graph Id";
        public string GraphModel = "Graph Model";
        public string HasBeenAddedToClipboard = "has been added to clipboard...";
        public string Height = "Height";
        public string HeightFromRadius = "Height From Radius";
        public string HeightRadiusFactor = "Height Radius Factor";
        public string Help = "Help";
        public string HelpResources = "Help Resources";
        public string Hide = "Hide";
        public string HideAnimationWillNotWork = "Hide animation will not work.";
        public string HideOnBackButton = "Hide on Back Button";
        public string HidePopup = "Hide Popup";
        public string HideProgressor = "Hide Progressor";
        public string HideUIPopupBy = "Hide UIPopup by";
        public string HideView = "Hide View";
        public string HideViews = "Hide UIView";
        public string HowToUse = "How To Use";
        public string IdleCheckInterval = "Idle Check Interval";
        public string IdleCheckIntervalDescription = "Time interval (in seconds) to check for idle Soundy Controllers to kill them";
        public string IgnoreListenerPause = "Ignore Listener Pause";
        public string IgnoreTimescale = "Ignore Timescale";
        public string Image = "Image";
        public string Images = "Images";
        public string ImageType = "Image Type";
        public string IncludeAudioClipNamesInSearch = "Include AudioClip names in search";
        public string Info = "Info";
        public string InputConnected = "Input Connected";
        public string InputConnections = "Input Connections";
        public string InputMode = "Input Mode";
        public string InputNotConnected = "Input Not Connected";
        public string Installed = "Installed";
        public string InstantAction = "Instant Action";
        public string InstantAnimation = "Instant Animation";
        public string Integrations = "Integrations";
        public string IsNotPrefab = "is not a prefab.";
        public string Key = "Key";
        public string KeyCode = "Key Code";
        public string Label = "Label";
        public string Labels = "Labels";
        public string Language = "Language";
        public string LastModified = "Last Modified";
        public string Left = "Left";
        public string ListenForAllGameEvents = "Listen for all GameEvents";
        public string ListenForAllUIButtons = "Listen for all UIButtons";
        public string ListenForAllUIDrawers = "Listen for all UIDrawers";
        public string ListenForAllUIViews = "Listen for all UIViews";
        public string ListeningForGameEvent = "Listening for GameEvent";
        public string ListIsEmpty = "List is empty...";
        public string Load = "Load";
        public string LoadBehavior = "Load Behavior";
        public string LoadedGraph = "Loaded Graph";
        public string LoadPreset = "Load Preset";
        public string LoadSceneBy = "Load Scene By";
        public string LoadSceneMode = "Load Scene Mode";
        public string LoadSceneNodeName = "Load Scene";
        public string LoadSelectedPresetAtRuntime = "Load selected preset at runtime.";
        public string LongTapDuration = "Long Tap Duration";
        public string LongTapDurationDescription = "Time duration, for a light touch on a touch-sensitive screen, to be considered a long tap (long press)";
        public string LoopSound = "Loop Sound";
        public string LoopType = "Loop Type";
        public string LoopView = "View Loop";
        public string Manual = "Manual";
        public string Material = "Material";
        public string Max = "Max";
        public string MaxAngle = "Max Angle";
        public string MaxHeightFactor = "Max Height Factor";
        public string MaxRadius = "Max Radius";
        public string MaxValue = "Max Value";
        public string MaxWidthFactor = "Max Width Factor";
        public string Min = "Min";
        public string MinAngle = "Min Angle";
        public string MinimumNumberOfControllers = "Minimum Number of Controllers";
        public string MinimumNumberOfControllersDescription = "The minimum number of Soundy Controllers that should be available in the Soundy Pooler, that will not get automatically killed even if they are killable";
        public string MinimumSize = "Minimum Size";
        public string MinValue = "Min Value";
        public string MissingPrefabReference = "Missing prefab reference";
        public string Move = "Move";
        public string MoveBy = "Move By";
        public string MoveDown = "Move Down";
        public string MoveFrom = "Move From";
        public string MoveTo = "Move To";
        public string MoveUp = "Move Up";
        public string Multiplier = "Multiplier";
        public string MuteAllSounds = "Mute All Sounds";
        public string Name = "Name";
        public string New = "New";
        public string NewCategory = "New Category";
        public string NewCategoryNameCannotBeEmpty = "Category name cannot be empty. Please enter a category name.";
        public string NewDatabase = "New Database";
        public string NewPopup = "New Popup";
        public string NewPreset = "New Preset";
        public string NewPresetNameCannotBeEmpty = "Preset name cannot be empty. Please enter a new preset name.";
        public string News = "News";
        public string NewSoundDatabase = "New Sound Database";
        public string No = "No";
        public string NoAnimationEnabled = "No Animation Enabled";
        public string NoAnimatorFound = "No Animator Found";
        public string Node = "Node";
        public string NodeId = "Node Id";
        public string NodeName = "Node Name";
        public string NodeNameTooltip = "The name of this node. Should be an unique name inside the graph.";
        public string Nodes = "Nodes";
        public string NodeState = "Node State";
        public string NodeWidth = "Node Width";
        public string NodySettings = "Nody Settings";
        public string NoGraphReferenced = "No graph referenced";
        public string NormalLoopAnimation = "Normal Loop Animation";
        public string NoSound = "No Sound";
        public string NoSoundsHaveBeenAdded = "No sounds have been added";
        public string NotConnected = "Not connected";
        public string Notes = "Notes";
        public string NoteworthyInformation = "Noteworthy information";
        public string NotInstalled = "Not Installed";
        public string NumberOfLoops = "Loops";
        public string Ok = "Ok";
        public string OnAnimationFinished = "OnAnimationFinished";
        public string OnAnimationStart = "OnAnimationStart";
        public string OnClick = "OnClick";
        public string OnDeselected = "OnDeselected";
        public string OnDisable = "OnDisable";
        public string OnDoubleClick = "OnDoubleClick";
        public string OnEnable = "OnEnable";
        public string OnEnter = "On Enter";
        public string OnEnterNode = "On Enter Node";
        public string OnExit = "On Exit";
        public string OnExitNode = "On Exit Node";
        public string OnFixedUpdate = "On FixedUpdate";
        public string OnLateUpdate = "On LateUpdate";
        public string OnLoadScene = "On Load Scene";
        public string OnLongClick = "OnLongClick";
        public string OnNodeFixedUpdate = "On Node FixedUpdate";
        public string OnNodeLateUpdate = "On Node LateUpdate";
        public string OnNodeUpdate = "On Node Update";
        public string OnPointerDown = "OnPointerDown";
        public string OnPointerEnter = "OnPointerEnter";
        public string OnPointerExit = "OnPointerExit";
        public string OnPointerUp = "OnPointerUp";
        public string OnRightClick = "OnRightClick";
        public string OnSceneLoaded = "On Scene Loaded";
        public string OnSelected = "OnSelected";
        public string OnTrigger = "OnTrigger";
        public string OnUpdate = "On Update";
        public string Open = "Open";
        public string OpenControlPanel = "Open Control Panel";
        public string OpenDatabase = "Open Database";
        public string OpenDrawer = "Open Drawer";
        public string Opened = "Opened";
        public string OpenedPosition = "Opened Position";
        public string OpenGraph = "Open Graph";
        public string OpenNody = "Open Nody";
        public string OpenSpeed = "Open Speed";
        public string OpenSubGraph = "Open SubGraph";
        public string OperationCannotBeUndone = "Operation cannot be undone.";
        public string OrientationDetectorDescription = "Detect the current screen orientation of the target device";
        public string OtherReferences = "Other References";
        public string OutputAudioMixerGroup = "Output Audio Mixer Group";
        public string OutputConnected = "Output Connected";
        public string OutputConnections = "Output Connections";
        public string OutputMixerGroup = "Output Mixer Group";
        public string OutputNotConnected = "Output Not Connected";
        public string Overlay = "Overlay";
        public string Override = "Override";
        public string OverrideColor = "Override Color";
        public string Overview = "Overview";
        public string OverviewZoom = "Overview Zoom";
        public string ParameterName = "Parameter Name";
        public string ParameterType = "Parameter Type";
        public string ParticleSystem = "Particle System";
        public string Paste = "Paste";
        public string PauseAllSounds = "Pause All Sounds";
        public string PercentageOfScreenZeroToOne = "Percentage Of Screen (0 - 1)";
        public string PitchSemitones = "Pitch (semitones)";
        public string Play = "Play";
        public string PlayAnimationInZeroSeconds = "Play the animation in zero seconds (instant action)";
        public string PlayMode = "Play Mode";
        public string PlaySound = "Play Sound";
        public string PleaseEnterNewName = "Please enter a new name.";
        public string PopupName = "Popup Name";
        public string PopupPrefab = "Popup Prefab";
        public string Portal = "Portal";
        public string PortalNodeName = "Portal Node";
        public string Prefix = "Prefix";
        public string PreserveAspect = "Preserve Aspect";
        public string PresetCategory = "Preset Category";
        public string PresetName = "Preset Name";
        public string PreviewAnimation = "Preview Animation";
        public string Progress = "Progress";
        public string Progressor = "Progressor";
        public string Progressors = "Progressors";
        public string ProgressTargets = "Progress Targets";
        public string PunchBy = "Punch By";
        public string Radius = "Radius";
        public string RadiusControlsHeight = "Radius Controls Height";
        public string RadiusControlsWidth = "Radius Controls Width";
        public string RadiusHeightFactor = "Radius Height Factor";
        public string RadiusWidthFactor = "Radius Width Factor";
        public string RandomDuration = "Random Duration";
        public string RandomNodeName = "Random";
        public string Range = "Range";
        public string RaycastTarget = "Raycast Target";
        public string Recent = "Recent";
        public string Refresh = "Refresh";
        public string RefreshDatabase = "Refresh Database";
        public string RegisterInterval = "Register Interval";
        public string Remove = "Remove";
        public string RemoveCategory = "Remove Category";
        public string RemovedDuplicateEntries = "Removed duplicate entries";
        public string RemovedEntriesWithNoName = "Removed entries with no name";
        public string RemovedEntriesWithNullPrefabs = "Removed entries with null prefabs";
        public string RemovedEntry = "Removed Entry";
        public string RemovedNullEntries = "Removed null entries";
        public string RemoveDuplicates = "Remove Duplicates";
        public string RemoveEmptyCategories = "Remove Empty Categories";
        public string RemoveEmptyEntries = "Remove Empty Categories";
        public string RemoveItem = "Remove Item";
        public string RemoveNullEntries = "Remove Null Entries";
        public string RemoveSymbolFromAllBuildTargetGroups = "Remove symbol from all the BuildTargetGroups?";
        public string Rename = "Rename";
        public string RenameCategory = "Rename Category";
        public string RenameCategoryDialogMessage = "All the UI settings (and code references) that use this category name, will not get automatically changed.";
        public string RenameGameObjectTo = "Rename GameObject to";
        public string RenameNodeTo = "Rename Node to";
        public string RenamePrefix = "Rename Prefix";
        public string RenameSoundDatabase = "Rename Sound Database";
        public string RenameSoundDatabaseDialogMessage = "All the UI settings (and code references) that use this database name, will not get automatically changed.";
        public string RenameSuffix = "Rename Suffix";
        public string RenameTo = "Rename to";
        public string Reset = "Reset";
        public string ResetAnimationSettings = "Reset Animation Settings";
        public string ResetClosedPosition = "Reset Closed Position";
        public string ResetDatabase = "Reset Database";
        public string ResetDelay = "Reset Delay";
        public string ResetOpenedPosition = "Reset Opened Position";
        public string ResetPosition = "Reset Position";
        public string ResetRoot = "Reset Root";
        public string ResetTrigger = "Reset Trigger";
        public string ResetValue = "Reset Value";
        public string ReversedProgress = "Reversed Progress";
        public string Right = "Right";
        public string Rotate = "Rotate";
        public string RotateBy = "Rotate By";
        public string RotateChildren = "Rotate Children";
        public string RotateFrom = "Rotate From";
        public string RotateMode = "Rotate Mode";
        public string RotateTo = "Rotate To";
        public string RuntimeOptions = "Runtime Options";
        public string RuntimePreset = "Runtime Preset";
        public string Save = "Save";
        public string SaveAs = "SaveAs";
        public string SavePreset = "Save Preset";
        public string Scale = "Scale";
        public string ScaleBy = "Scale By";
        public string ScaleFrom = "Scale From";
        public string ScaleTo = "Scale To";
        public string Scene = "Scene";
        public string SceneActivationDelay = "Scene Activation Delay";
        public string SceneBuildIndex = "Scene Build Index";
        public string SceneLoad = "Scene Load";
        public string SceneName = "Scene Name";
        public string SceneUnload = "Scene Unload";
        public string ScriptingDefineSymbol = "Scripting Define Symbol";
        public string Search = "Search";
        public string SearchForCategories = "Search for categories";
        public string SearchForDatabases = "Search for databases";
        public string SearchForUIPopupLinks = "Search for UIPopup links";
        public string Seconds = "Seconds";
        public string SecondsDelay = "Seconds Delay";
        public string Select = "Select";
        public string SelectButton = "Select Button";
        public string SelectedLoopAnimation = "Selected Loop Animation";
        public string SelectSwipeDirection = "Select Swipe Direction";
        public string Send = "Send";
        public string SendGameEvent = "Send Game Event";
        public string SendGameEvents = "Send Game Events";
        public string SetActiveNode = "Set Active Node";
        public string SetAsSoundName = "Set as Sound Name";
        public string SetBoolValueTo = "Set Bool value to";
        public string SetFloatValueTo = "Set Float value to";
        public string SetIntValueTo = "Set Int value to";
        public string SetPosition = "Set Position";
        public string SetTargetGameObject = "Set Target GameObject";
        public string Settings = "Settings";
        public string SetUIButtonToListenFor = "Set an UIButton to listen for";
        public string SetUIDrawerToListenFor = "Set an UIDrawer to listen for";
        public string SetUIViewToListenFor = "Set an UIView to listen for";
        public string SetValue = "Set Value";
        public string Show = "Show";
        public string ShowAnimationWillNotWork = "Show animation will not work.";
        public string ShowCurveModifier = "Show Curve Modifier";
        public string ShowNodeNotes = "Show Node Notes";
        public string ShowNodeNotesDescription = "Toggle visibility of node notes";
        public string ShowPopup = "Show Popup";
        public string ShowProgressor = "Show Progressor";
        public string ShowView = "Show View";
        public string ShowViews = "Show UIView";
        public string Simulate = "Simulate";
        public string SocialLinks = "Social Links";
        public string Socket = "Socket";
        public string Sort = "Sort";
        public string SortDatabase = "Sort Database";
        public string SortingSteps = "Sorting Steps";
        public string Sound = "Sound";
        public string SoundAction = "Sound Action";
        public string SoundDatabases = "Sound Databases";
        public string SoundName = "Sound Name";
        public string SoundNodeName = "Sound Node";
        public string Sounds = "Sounds";
        public string SoundSource = "Sound Source";
        public string Soundy = "Soundy";
        public string SoundyDatabase = "Soundy Database";
        public string SoundySettings = "Soundy Settings";
        public string SourceImage = "Source Image";
        public string SourceName = "Source Name";
        public string Sources = "Sources";
        public string Spacing = "Spacing";
        public string SpatialBlend = "Spatial Blend (0-1)";
        public string Speed = "Speed";
        public string StartAngle = "Start Angle";
        public string StartDelay = "Start Delay";
        public string StartNodeName = "Start";
        public string StopAllSounds = "Stop All Sounds";
        public string StopAnimation = "Stop Animation";
        public string StopBehavior = "Stop Behavior";
        public string StopSound = "Stop Sound";
        public string SubGraph = "SubGraph";
        public string SubGraphNodeName = "SubGraph";
        public string Suffix = "Suffix";
        public string SupportEmail = "Support Email";
        public string SwipeDirection = "Swipe Direction";
        public string SwipeLength = "Swipe Length";
        public string SwipeLengthDescription = "Minimum travel distance, for a sliding a finger or a stylus pen across a touch-sensitive screen, to be considered a swipe";
        public string SwitchBackNodeName = "Switch Back";
        public string Target = "Target";
        public string TargetAnimator = "Target Animator";
        public string TargetAnimatorDoesNotHaveAnAnimatorController = "Target Animator does not have an Animator Controller";
        public string TargetAnimatorDoesNotHaveAnyParameters = "Target Animator does not have any parameters";
        public string TargetCanvas = "Target Canvas";
        public string TargetFsm = "Target FSM";
        public string TargetGameObject = "Target GameObject";
        public string TargetLabel = "Target Label";
        public string TargetMixer = "Target Mixer";
        public string TargetOrientation = "Target Orientation";
        public string TargetProgress = "Target Progress";
        public string TargetTimeScale = "Target Time Scale";
        public string TargetValue = "Target Value";
        public string TargetVariable = "Target Variable";
        public string Text = "Text";
        public string TextLabel = "Text Label";
        public string TextMeshPro = "TextMeshPro";
        public string TextMeshProLabel = "TextMeshPro Label";
        public string Time = "Time";
        public string TimeScaleNodeName = "TimeScale";
        public string ToggleComponentBehaviors = "Toggle component Behaviors";
        public string ToggleLabel = "Toggle Label";
        public string ToggleOFF = "Toggle OFF";
        public string ToggleON = "Toggle ON";
        public string ToggleProgressor = "Toggle Progressor";
        public string ToggleSettings = "Toggle Settings";
        public string ToggleSupportForThirdPartyPlugins = "Toggle support for third party plug-ins";
        public string Tolerance = "Tolerance";
        public string TouchySettings = "Touchy Settings";
        public string TriggerAction = "Trigger Action";
        public string TriggerEventsAfterAnimation = "Trigger events after animation";
        public string TriggerName = "Trigger Name";
        public string TriggerValue = "Trigger Value";
        public string UIDrawerNodeName = "UIDrawer";
        public string UINodeNodeName = "UI Node";
        public string UIPopupDatabase = "UIPopup Database";
        public string UnityEvent = "Unity Event";
        public string UnityEvents = "Unity Events";
        public string UnloadSceneNodeName = "Unload Scene";
        public string UnmuteAllSounds = "Unmute All Sounds";
        public string UnpauseAllSounds = "Unpause All Sounds";
        public string Up = "Up";
        public string UpdateContainer = "Update Container";
        public string UpdateEffect = "Update Effect";
        public string TriggerActions = "Trigger Actions";
        public string Is = "is";
        public string And = "and";
        public string To = "to";
        public string UpdateOnHide = "Update on Hide";
        public string UpdateOnShow = "Update on Show";
        public string UpdatePopupName = "Update Popup Name";
        public string UpdatePopupPrefab = "Update Popup Prefab";
        public string UpdateValue = "Update Value";
        public string UseBackButtonDescription = "Use the 'Back' button functionality";
        public string UseCustomFromAndTo = "Custom From & To";
        public string UsefulLinks = "Useful Links";
        public string UseLogarithmicConversion = "Use Logarithmic Conversion";
        public string UseMultiplier = "Use Multiplier";
        public string UseUnscaledTime = "Use Unscaled Time";
        public string Value = "Value";
        public string Version = "Version";
        public string Vibrato = "Vibrato";
        public string ViewCategory = "View Category";
        public string ViewName = "View Name";
        public string VirtualButton = "Virtual Button";
        public string VolumeDb = "Volume (dB)";
        public string WaitFor = "Wait For";
        public string WaitForAnimationToFinish = "Wait for Animation to Finish";
        public string WaitForSceneToUnload = "Wait for Scene to Unload";
        public string WaitNodeName = "Wait";
        public string Warning = "Warning";
        public string Weight = "Weight";
        public string When = "When";
        public string WhenClosed = "When Closed";
        public string WhenUIDrawerIsClosed = "When this UIDrawer is closed";
        public string WhenUIPopupIsHiddenDisable = "When this UIPopup is hidden disable the Container";
        public string WhenUIViewIsHiddenDisable = "When UIView is hidden disable";
        public string WholeNumbers = "Whole Numbers";
        public string Width = "Width";
        public string X = "X";
        public string Y = "Y";
        public string Yes = "Yes";
        public string YouAreResponsibleToUpdateYourCode = "You are responsible to update your code and the UI settings.";
        public string YouCanOnlyAddPrefabsToDatabase = "You can only add prefabs to the database";
        public string YouTube = "YouTube";


        [Header("Info Messages")]
        public string MissingTargetFsmMessage = "Missing Target Fsm";

        public string SelectListenerToActivateMessage = "Select at least one listener";

        public string HowToUsePlaymakerEventDispatcherMessage = "This dispatcher auto targets the first FSM on this GameObject." +
                                                                "\n" +
                                                                "You can override that and reference the FSM you want to target." +
                                                                "\n\n" +
                                                                "For this dispatcher to work with the target Playmaker FSM you have to create FSM Events named exactly as the Game Events or Button Names that you want to listen for and react to." +
                                                                "The FSM Events, Game Events and Button Names are case sensitive." +
                                                                "\n\n" +
                                                                "To dispatch Game Events, you have to create, in the target FSM, Events named exactly as the Game Event commands you wants to catch." +
                                                                "\n\n" +
                                                                "To dispatch Button Clicks, you have to create, in the target FSM, Events named exactly as the Button Names you wants to catch.";

        public string MissingDrawerNameTitle = "Missing Drawer Name";
        public string MissingDrawerNameMessage = "Please enter a drawer name.";
        public string MissingGameEventTitle = "Missing Game Event";
        public string MissingGameEventMessage = "Please enter a game event.";
        public string MissingSceneNameTitle = "Missing Scene Name";
        public string MissingSceneNameMessage = "Please enter a scene name.";
        public string WrongSceneBuildIndexTitle = "Wrong Scene Build Index";
        public string WrongSceneBuildIndexMessage = "Check the 'Scenes in Build' in the 'Build Settings' window.";
        public string DoubleClickNodeToOpenSubGraphMessage = "In the Nody Graph Window, you can double click on a Sub-Graph Node's header or footer to open it's referenced SubGraph";
        public string DuplicateNodeMessage = "There is another node with the same node name in this graph. \n\nPlease rename this node to an unique name.";
        public string DuplicateNodeTitle = "Duplicate Node Name";
        public string NoGraphReferencedMessage = "Please reference a Graph asset.";
        public string NoGraphReferencedTitle = "No Graph asset has been referenced";
        public string NoSourceConnectedMessage = "Please connect at least one Source Input and Output Sockets pair, that will be considered as one of the Sources Paths.";
        public string NoSourceConnectedTitle = "No Source";
        public string NoSubGraphReferencedMessage = "Please reference a Sub-Graph asset.";
        public string NoSubGraphReferencedTitle = "No Graph asset has been referenced";
        public string NoTargetConnectedMessage = "Please connect the Target Input and Output Sockets to create the Target Path.";
        public string NoTargetConnectedTitle = "No Target";
        public string NotConnectedMessage = "This node should to be connected in order for the graph to work as expected. \n\nPlease connect this node to another other node.";
        public string NotConnectedTitle = "Node is not connected";
        public string NotListeningForAnyGameEventMessage = "Please enter a non-empty GameEvent to fix this issue.";
        public string NotListeningForAnyGameEventTitle = "Not listening for any GameEvent.";
        public string NotSendingAnyGameEventMessage = "Please enter a non-empty GameEvent to fix this issue.";
        public string NotSendingAnyGameEventTitle = "Not sending any GameEvents";

        public string ProgressTargetAnimatorParameterInfo = "The ParameterName variable name should be a float parameter defined in the Animator." +
                                                            "\n\nThe float parameter should be set as the 'Normalized Time' parameter for the target Animation." +
                                                            "\n\nSee documentation and videos for more details.";

        public string ReferencedGraphIsNotSubGraphMessage = "Please reference a Graph that is set to work as a Sub-Graph. \n\nA Graph that is set to work as a Sub-Graph will have an Enter Node and an Exit Node, unlike a Graph that has only a Start Node.";
        public string ReferencedGraphIsNotSubGraphTitle = "The referenced Graph is NOT a Sub-Graph";
        public string ReferencedGraphIsSubGraphMessage = "Please reference a Graph that is NOT set to work as a Sub-Graph. \n\nA Graph that is set to work as a Sub-Graph will have an Enter Node and an Exit Node, unlike a Graph that has only a Start Node.";
        public string ReferencedGraphIsSubGraphTitle = "The referenced Graph is a Sub-Graph";
        public string SomeProgressTargetsGetUpdatedOnlyInPlayMode = "Some Progress Targets (like the Animator) get updated only in Play Mode. Thus the simulation will not work in Edit Mode for those targets.";
        public string SupportForMasterAudioNotEnabled = "Support for Master Audio has not been enabled in the Control Panel";
        public string SupportForPlaymakerNotEnabled = "Support for Playmaker has not been enabled in the Control Panel";
        public string SupportForTextMeshProNotEnabled = "Support for TextMeshPro has not been enabled in the Control Panel";
        public string ThisClassShouldBeExtended = "This class should not be used like this. Use another class that inherits from this one to work as expected.";
        public string UnnamedNodeMessage = "This node does not have a name. \n\nPlease enter an unique name for this node.";
        public string UnnamedNodeTitle = "Unnamed Node";
    }
}