﻿using System;
using UnityEngine;
//using Newtonsoft.Json;
using System.Text;

namespace Seven
{
    public static class ConverterExtend
    {
        private const string dateTimeFormat = dateFormat + " HH:mm:ss:ff";
        private const string dateFormat = "dd-MM-yyyy";
        //private static JsonSerializerSettings JsonSettings = new JsonSerializerSettings() { NullValueHandling = NullValueHandling.Include, TypeNameHandling = TypeNameHandling.All, DefaultValueHandling = DefaultValueHandling.Include };

        public static bool ToBoolean(this object obj)
        {
            try
            {
                return System.Convert.ToBoolean(obj);
            }
            catch (System.Exception ex)
            {
                ex.Message.Log();
            }

            return false;
        }
        public static int ToInt32(this object obj)
        {
            try
            {
                return System.Convert.ToInt32(obj);
            }
            catch (System.Exception ex)
            {
                ex.Message.Log();
            }

            return 0;
        }
        public static long ToInt64(this object obj)
        {
            return System.Convert.ToInt64(obj);
        }
        public static ulong ToUInt64(this object obj)
        {
            return System.Convert.ToUInt64(obj);
        }
        public static float ToSingle(this object obj)
        {
            try
            {
                return System.Convert.ToSingle(obj);
            }
            catch (System.Exception ex)
            {
                ex.Message.Log();
            }

            return 0.0f;
        }

        public static Texture2D ToTexture2D(this byte[] data)
        {
            if (data == null)
                return null;

            Texture2D result = new Texture2D(1, 1, TextureFormat.ARGB32, false);
            if (result.LoadImage(data))
            {
                result.Apply();
            }

            return result;
        }
        public static Texture2D ToTexture2D(this Sprite sprite)
        {
            var texture = new Texture2D((int)sprite.rect.width, (int)sprite.rect.height, TextureFormat.ARGB32, false);
            var pixels = sprite.texture.GetPixels((int)sprite.rect.x, (int)sprite.rect.y, (int)sprite.rect.width, (int)sprite.rect.height);
            texture.SetPixels(pixels);
            texture.Apply();
            return texture;
        }
        public static Texture2D ToTexture2D(this RenderTexture target, ref Texture2D tex)
        {
            if (tex == null)
                tex = new Texture2D(target.width, target.height, TextureFormat.RGB24, false);

            RenderTexture.active = target;
            tex.ReadPixels(new Rect(0, 0, target.width, target.height), 0, 0);
            tex.Apply();
            return tex;
        }
        public static Sprite ToSprite(this Texture2D texture, Rect rect, Vector2 pivot, float pixelPerUnit = 100)
        {
            if (texture == null)
            {
                return null;
            }
            else
            {
                return Sprite.Create(texture, rect, pivot, pixelPerUnit);
            }
        }
        public static Sprite ToSprite(this Texture2D texture, Rect rect, float pixelPerUnit = 100)
        {
            if (texture == null)
            {
                return null;
            }
            else
            {
                return Sprite.Create(texture, rect, Vector2.one * 0.5f, pixelPerUnit);
            }
        }
        public static Sprite ToSprite(this Texture2D texture, float pixelPerUnit = 100)
        {
            return ToSprite(texture, new Rect(0, 0, texture.width, texture.height), pixelPerUnit);
        }       
        public static DateTime ToDateTime(this string timeString, string format = dateTimeFormat)
        {
            return DateTime.ParseExact(timeString, format, null);
        }
        public static DateTime ToDate(this string timeString, string format = dateFormat)
        {
            return DateTime.ParseExact(timeString, format, null);
        }
        // public static string ToJson(this object target,bool pretty = true)
        // {
        //     return JsonConvert.SerializeObject(target, pretty ? Formatting.Indented : Formatting.None, JsonSettings);
        // }         
        public static string GetString(this byte[] bytes)
        {
            return Encoding.ASCII.GetString(bytes);
        }
        public static byte[] ToBytes(this string str)
        {
            return Encoding.ASCII.GetBytes(str);
        }

        // public static T FromJson<T>(this string json) where T : class
        // {
        //     return JsonConvert.DeserializeObject<T>(json, JsonSettings);
        // }

        public static string ToStringDateTime(this DateTime dateTime, string format = dateTimeFormat)
        {
            return dateTime.ToString(format);
        }
        public static string ToStringDate(this DateTime dateTime, string format = dateFormat)
        {
            return dateTime.ToString(format);
        }
    }
}
