﻿using UnityEngine;

namespace Seven
{
    public static class GameObjectExtend
    {
        public static T GetOrAddComponent<T>(this GameObject target, bool forceAddIfNotExist) where T : Component
        {
            var findComponent = target.GetComponent<T>();
            if (findComponent == null)
            {
                findComponent = target.AddComponent<T>();
            }

            return findComponent;
        }
        public static void Destroy(this GameObject target)
        {
            if(target.IsNull())
                return;

            UnityEngine.Object.Destroy(target);
            target = null;
        }
        public static void DontDestroyOnLoad(this GameObject obj)
        {
            UnityEngine.Object.DontDestroyOnLoad(obj);
        }
        public static void RemoveComponent<T>(this GameObject obj) where T : Component
        {
            if (obj == null)
                return;

            T component = obj.GetComponent<T>();
            if (component != null)
            {
                GameObject.Destroy(component);
            }

        }
        public static void Show(this UnityEngine.GameObject obj)
        {
            if (obj == null)
                return;

            if (!obj.activeInHierarchy)
            {
                obj.SetActive(true);
            }
        }
        public static void Hide(this UnityEngine.GameObject obj)
        {
            if (obj == null)
                return;

            if (obj.activeInHierarchy)
            {
                obj.SetActive(false);
            }
        }
    }
}
