﻿using UnityEngine;
using Sirenix.OdinInspector;
using System;

namespace Seven.Utility
{
    [HideMonoScript]
    [AddComponentMenu("Seven/Utils/Input/PinchZoom")]
    [RequireComponent(typeof(Camera))]
    public class PinchZoom : MonoBehaviour
    {
        #region field
        const string grpConfig = "Config";
        const string grpRuntime = "Runtime";
        #endregion

        #region Field
        [SerializeField,FoldoutGroup(grpConfig)] 
        private bool _useScroll,_canZoom = true;
        
        [SerializeField,FoldoutGroup(grpConfig)] 
        private float _zoomSpeed = 0.5f;
        
        [SerializeField,FoldoutGroup(grpConfig)] 
        private Vector2 _minMaxZoom = new Vector2(1, 100);
        #endregion

        #region Property
        public bool canZoom {
            get => _canZoom;
            set => _canZoom = value;
        }
        public new  Camera camera
        {
            get
            {
                return _camera;
            }
        }
        public float zoomSpeed
        {
            get => _zoomSpeed;
            set => _zoomSpeed = value;
        }      
        public Vector2 minMaxZoom
        {
            get => _minMaxZoom;
            set => _minMaxZoom = value;
        }
        [ShowInInspector,FoldoutGroup(grpRuntime)]
        public float zoomValue{
            get
            {
                if(_camera.IsNull())
                    return 0;

                return _camera.orthographic? _camera.orthographicSize : _camera.fieldOfView;
            }

            set
            {
                value = Mathf.Clamp(value,minMaxZoom.x,minMaxZoom.y);
            
                if(_camera.orthographic)
                    _camera.orthographicSize  = value;
                else
                    _camera.fieldOfView = value;
            }
        }
        [ShowInInspector,FoldoutGroup(grpRuntime)]
        public float zoomScale{
            get
            {
                var range = minMaxZoom.y - minMaxZoom.x;
                return (zoomValue - minMaxZoom.x) / range; 
            }
            set
            {
                value = Mathf.Clamp01(value);
                var range = _minMaxZoom.y - _minMaxZoom.x;
                var zoomValue = _minMaxZoom.x + (value * range);
                this.zoomValue = zoomValue;
            }
        }
        #endregion
        
        public event Action<float> onZooming;

        
        #region Private
        private float _deltaMagnitudeDiff;
        private Camera _camera;
        private float _prevZoom;
        private float _initZoom;
        #endregion

        private void Start()
        {
            if (_camera == null)
                _camera = GetComponent<Camera>();

            if(_camera != null)
               _prevZoom = _camera.orthographic? _camera.orthographicSize : _camera.fieldOfView;

            _initZoom = _prevZoom;
        }
        private void Reset()
        {
            if (_camera == null)
                _camera = GetComponent<Camera>();

            if (_camera == null)
                return;
        }
        private void Update()
        {
            if(!_canZoom)
                return;

            var inputed = false;
            // If there are two touches on the device...
            if (Input.touchCount == 2)
            {
                // Store both touches.
                Touch touchZero = Input.GetTouch(0);
                Touch touchOne = Input.GetTouch(1);

                // Find the position in the previous frame of each touch.
                Vector2 touchZeroPrevPos = touchZero.position - touchZero.deltaPosition;
                Vector2 touchOnePrevPos = touchOne.position - touchOne.deltaPosition;

                // Find the magnitude of the vector (the distance) between the touches in each frame.
                float prevTouchDeltaMag = (touchZeroPrevPos - touchOnePrevPos).magnitude;
                float touchDeltaMag = (touchZero.position - touchOne.position).magnitude;

                // Find the difference in the distances between each frame.
                _deltaMagnitudeDiff = prevTouchDeltaMag - touchDeltaMag;

                inputed = true;            
            }
            else if (_useScroll && Input.mouseScrollDelta.y != 0)
            {
                _deltaMagnitudeDiff = Input.mouseScrollDelta.y * 10;  
                inputed = true;
            }

            if(!inputed)
                return;

            var zoomSize = _camera.orthographic ? _camera.orthographicSize : _camera.fieldOfView;
            zoomSize += _deltaMagnitudeDiff * _zoomSpeed;
            zoomSize = Mathf.Clamp(zoomSize ,_minMaxZoom.x, _minMaxZoom.y);

            if(_prevZoom != zoomSize)
            {
                _prevZoom = zoomSize;

                if(!canZoom)
                    return;

                if(_camera.orthographic)
                    _camera.orthographicSize = zoomSize;
                else
                    _camera.fieldOfView = zoomSize;

                onZooming?.Invoke(_prevZoom);
            }
        }
    
        public void ResetZoom(){
            if(_camera ==null)
                return;

            zoomValue = _initZoom;
        }
    }
}