﻿using UnityEngine;
using Sirenix.OdinInspector;

namespace Seven.Utility
{
    [HideMonoScript,AddComponentMenu("Seven/Utils/Input/ScreenTouchRotate")]
    public class ScreenTouchRotate : MonoBehaviour
    {
        #region Field
        [SerializeField] private float _pinchTurnRatio = Mathf.PI / 2;
        [ShowInInspector] private float _angle;
        [ShowInInspector] private float _angleDelta;
        #endregion

        #region Property
        public float pinchTurnRatio {
            get
            {
                return _pinchTurnRatio;
            }
        }
        public float angle
        {
            get
            {
                return _angle;
            }
        }
        public float angleDelta
        {
            get
            {
                return _angleDelta;
            }
        }
        #endregion

        #region Private
        private float _prevAngle;    
        #endregion

        private void LateUpdate()
        {
            if (Input.touchCount != 2)
                return;

            Touch touch1 = Input.touches[0];
            Touch touch2 = Input.touches[1];
            _angle  = Angle(touch1.position, touch2.position);
            _prevAngle = Angle(touch1.position - touch1.deltaPosition,touch2.position - touch2.deltaPosition);
            _angleDelta = Mathf.DeltaAngle(_prevAngle, _angle) * _pinchTurnRatio;
        }
        private float Angle(Vector2 pos1, Vector2 pos2)
        {
            Vector2 from = pos2 - pos1;
            Vector2 to = new Vector2(1, 0);

            float result = Vector2.Angle(from, to);
            Vector3 cross = Vector3.Cross(from, to);

            if (cross.z > 0)
            {
                result = 360f - result;
            }

            return result;
        }

        void OnDrawGizmos() { }
    }
}