﻿using UnityEngine;
using Sirenix.OdinInspector;
using System;

namespace Seven.Utility
{
    [HideMonoScript]
    [AddComponentMenu("Seven/Utils/Input/SwipeInput")]
    public class SwipeInput : Singleton<SwipeInput>
    {
        #region Events
        public event Action onSwipeUp,onSwipeDown,onSwipeLeft,onSwipeRight;
        #endregion

        #region Field
        [SerializeField] private float deadZone = 80;
        #endregion

        protected override bool dontDestroy => true;
        protected override HideFlags hideFlags => HideFlags.HideInHierarchy;

        #region Private
        private bool _tap, _swipeLeft, _swipeRight, _swipeUp, _swipeDown;
        private bool _isDragging = false;
        private Vector2 _startTouch, _swipeDelta; 
        #endregion

        private void Update()
        {
            _tap = _swipeLeft = _swipeRight = _swipeUp = _swipeDown = false;

            #region Standalone Input
            if (Input.GetMouseButtonDown(0))
            {
                _tap = true;
                _isDragging = true;
                _startTouch = Input.mousePosition;
            }
            else if (Input.GetMouseButtonUp(0))
            {
                _isDragging = false;
                Reset();
            }
            #endregion

            #region Mobile Input
            if (Input.touchCount > 0)
            {
                Touch touch = Input.touches[0];

                if (touch.phase == TouchPhase.Began)
                {
                    _tap = true;
                    _isDragging = true;
                    _startTouch = Input.touches[0].position;
                }
                else if (touch.phase == TouchPhase.Ended || touch.phase == TouchPhase.Canceled)
                {
                    _isDragging = false;
                    Reset();
                }
            }
            #endregion

            //Calculate distance
            _swipeDelta = Vector2.zero;
            if (_isDragging)
            {
                if (Input.touchCount > 0)
                {
                    _swipeDelta = Input.touches[0].position - _startTouch;
                }
                else
                {
                    _swipeDelta = (Vector2)Input.mousePosition - _startTouch;
                }
            }

            //Did we cross the deadzone
            if (_swipeDelta.magnitude > deadZone)
            {
                float x = _swipeDelta.x;
                float y = _swipeDelta.y;

                if (Mathf.Abs(x) > Mathf.Abs(y))
                {
                    //Left Or Right

                    if (x < 0)
                    {
                        _swipeLeft = true;
                        onSwipeLeft?.Invoke();
                    }
                    else
                    {
                        _swipeRight = true;
                        onSwipeRight?.Invoke();
                    }
                }
                else
                {
                    //Up or down
                    if (y < 0)
                    {
                        _swipeDown = true;
                        onSwipeDown?.Invoke();
                    }
                    else
                    {
                        _swipeUp = true;
                        onSwipeUp.Invoke();
                    }
                }

                Reset();
            }

        }
        private void Reset()
        {
            _startTouch = _swipeDelta = Vector2.zero;
            _isDragging = false;
        }

        void OnDrawGizmos() { }
    }
}